/*---------------------------------------------------------------------------*
Project: NitroWiFi - shared - demos
File: wcm_control.h

Copyright 2005.2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: wcm_control.h,v $
Revision 1.9  03/10/2006 09:22:42  kitase_hirotake
  INDENT SOURCE

Revision 1.8  2005/11/02 04:13:42  terui
Fix comment.

Revision 1.7  09/13/2005 13:13:29  terui
Added DisconnectAP function.

Revision 1.6  09/08/2005 10:42:38  adachi_hiroaki
Made it possible to get the number of round scans. Also made changes such that debug display within the callback function only appears once.

Revision 1.5  09/05/2005 04:00:49  terui
Revised such that scan method and power management mode can be changed externally.

Revision 1.4  2005/08/29 06:39:02  yosizaki
add SetWcmManualConnect(), ConnectAP().

Revision 1.3  08/22/2005 12:10:19  yasu
revised for reference to SiteDefs

Revision 1.2  07/27/2005 08:36:22  terui
Add SetWcmThreadPriority API

Revision 1.1  07/21/2005 08:38:16  terui
Added new source files common to samples

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#ifndef SHARED_WCM_CONTROL_H_
#define SHARED_WCM_CONTROL_H_

#include "ap_info.h"

#ifdef __cplusplus

extern "C" {
#endif

/*===========================================================================*/

/*---------------------------------------------------------------------------*
Name: InitWcmControl

Description: Initializes internal variables and launches the WCM control thread.
Nothing is performed if the thread has already been launched.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    InitWcmControl(void);
void    InitWcmControlByApInfo(const WcmControlApInfo* ptr);

/*---------------------------------------------------------------------------*
Name: InitWcmApInfo

Description: Gets access point connection parameters from environment setting values
and initializes.

Arguments: apinfo   connection parameter
apclass  access point class name

Returns: TRUE   success
FALSE  failure
 *---------------------------------------------------------------------------*/
BOOL    InitWcmApInfo(WcmControlApInfo* apinfo, const char* apclass);

/*---------------------------------------------------------------------------*
Name: TerminateWcmControl

Description: Ends the WCM control thread.
Nothing is performed if the thread has not been launched.
NOTICE: As this function only serves as a trigger to end the thread,
be aware that it will return without waiting for the thread itself to end.
If you want to wait for the thread itself to end, you will need to call
OS_JoinThread after calling OS_SendMessage.
In addition, if the WCM_GetPhase is set to return WCM_PHASE_NULL,
the thread has ended.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    TerminateWcmControl(void);

/*---------------------------------------------------------------------------*
Name: SetWcmThreadPriority

Description: Changes the priority of the thread performing WCM library state management.

Arguments: priority    -   Specifies a thread priority between 0 and 31.

Returns: u8  -   Returns the priority setting prior to being changed.
Returns 0xff if for some reason the change to priority fails.
 *---------------------------------------------------------------------------*/
u8      SetWcmThreadPriority(u8 priority);

/*---------------------------------------------------------------------------*
Name: GetWcmSearchAroundCount

Description: Returns the number of times all channels have been scanned.
0 immediately after StartUp.

Arguments: None.

Returns: How many times scanned?
 *---------------------------------------------------------------------------*/
u32     GetWcmSearchAroundCount(void);

/*---------------------------------------------------------------------------*
Name: SetWcmManualConnectMode

Description: Settings to select a connection manually from the AP list.
Default setting is FALSE (auto-connection).
Need to call before InitWcmControl().

Arguments: enable -   TRUE if manual connection.

Returns: None.
 *---------------------------------------------------------------------------*/
void    SetWcmManualConnect(BOOL enable);

/*---------------------------------------------------------------------------*
Name: ConnectAP

Description: connects to the specified BssDesc AP

Arguments: bd     -   Pointer to the structure indicated by the connection AP.

Returns: None.
 *---------------------------------------------------------------------------*/
void    ConnectAP(const WMBssDesc* pbd);

/*---------------------------------------------------------------------------*
Name: DisconnectAP

Description: attempts to disconnect from the AP connection

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    DisconnectAP(void);

/*---------------------------------------------------------------------------*
Name: SetWcmScanType

Description: Sets the scan method for searching for APs.
The default is the active scan method.
Need to call before InitWcmControl().

Arguments: type    -   Specifies the WCM_OPTION_SCANTYPE_* constant.

Returns: None.
 *---------------------------------------------------------------------------*/
void    SetWcmScanType(u32 type);

/*---------------------------------------------------------------------------*
Name: SetWcmPowerMode

Description: Specifies the power management method when communicating with an AP.
The default is the ordinary active method.
Need to call before InitWcmControl().

Arguments: mode    -   Specifies the WCM_OPTION_POWER_* constant.

Returns: None.
 *---------------------------------------------------------------------------*/
void    SetWcmPowerMode(u32 mode);

/*===========================================================================*/
#ifdef __cplusplus

}       /* extern "C" */
#endif

#endif /* SHARED_WCM_CONTROL_H_ */

/*---------------------------------------------------------------------------*
End of file
 *---------------------------------------------------------------------------*/
