/*---------------------------------------------------------------------------*
Project: NitroWiFi - SOC - demos - dhcp_monkey
File: dhcp_monkey.c

Copyright 2005.2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: dhcp_monkey.c,v $
Revision 1.2  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.1  12/22/2005 04:28:44  adachi_hiroaki
SOC version of dhcp_monkey

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include <nitroWiFi.h>
#include "wcm_control.h"
#include <sitedefs.h>

#define CLASS_AP    "WiFi.LAN.1.AP.1"

static void         Init(void);

/*---------------------------------------------------------------------------*
MAIN
 *---------------------------------------------------------------------------*/
static void         Heap_Setup(void);
static void*        myAlloc(u32 name, s32 size);
static void         myFree(u32 name, void* ptr, s32 size);
static SOCConfig    myConfig;

static WcmControlApInfo apInfo;

void NitroMain(void)
{
    int result;
    int n;

    OS_Init();
    OS_InitTick();
    OS_InitAlarm();
    RTC_Init();
    Heap_Setup();
    (void)OS_EnableIrq();
    (void)OS_EnableInterrupts();

    SiteDefs_Init();

    MI_CpuClear8(&myConfig, sizeof(myConfig));
    myConfig.flag = SOC_FLAG_DHCP;  // Use DHCP
    myConfig.alloc = myAlloc;
    myConfig.free = myFree;

// requested_ip developed as a strategy in response to the using up of the DHCP IP memory
// when the power is frequently turned on and off at the specified access point.
// The IP for which a tentative connection is made is saved into backup memory
// and that is specified here. This allows us to get the same IP address again.
// If it can be set, it should be.
// SOCL_SetRequestedIP(xxx.xxx.xxx.xxx);
    if (!InitWcmApInfo(&apInfo, CLASS_AP))
    {
        OS_Panic("Invalid AP Class....");
    }

    for (n = 0;; n++)
    {
        OS_TPrintf("################################ DHCP Count=%d\n", n);

// wireless connection
        InitWcmControlByApInfo(&apInfo);

// lower the main below the priority (24) for the WCM control thread
// or increase the WCM control thread priority in SetWcmThreadPrority()
// after calling InitWcmControl()
// You can also pass clear controls to WCM during main thread processing.
// Each will have its pros and cons, so the method you use
// will depend on the application.
// It is possible that a process will fail during WCM-related processes
// if the thread priority remains lowered after returning to a normal communication state.
// In addition, it may also be difficult to indicate regular calls to OS_Sleep, etc.,
// if the code is not in a loop structure for the method passing
// controls clearly to the WCM.
//(void)SetWcmThreadPriority(10);
        OS_TPrintf("LINK UP....\n");
        while (WCM_GetPhase() != WCM_PHASE_DCF)
        {
// when the priority for the main thread is higher than that of the WCM thread
// you need to clearly vacate the process
            OS_Sleep(100);
        }

        OS_TPrintf("DHCP....\n");

        (void)SOC_ClearConfigError(NULL);
        result = SOC_Startup(&myConfig);

        while (SOC_GetHostID() == 0 && SOC_GetConfigError(NULL) == 0)
        {
// when the priority for the main thread is higher than that of the WCM thread
// you need to clearly vacate the process
            OS_Sleep(100);
        }

        if (SOC_GetHostID() != 0)
        {
            int retry = 0;

            OS_TPrintf("IP      = %3d.%3d.%3d.%3d\n", CPS_CV_IPv4(SOCL_GetHostID()));
            OS_TPrintf("NetMask = %3d.%3d.%3d.%3d\n", CPS_CV_IPv4(CPSNetMask));
            OS_TPrintf("GW IP   = %3d.%3d.%3d.%3d\n", CPS_CV_IPv4(CPSGatewayIp));
            OS_TPrintf("DNS[0]  = %3d.%3d.%3d.%3d\n", CPS_CV_IPv4(CPSDnsIp[0]));
            OS_TPrintf("DNS[1]  = %3d.%3d.%3d.%3d\n", CPS_CV_IPv4(CPSDnsIp[1]));

            OS_TPrintf("Resolving....\n");
            {
                SOCHostEnt*     hotEnt = SOC_GetHostByName("www.nintendo.co.jp");
                if (hotEnt)
                {
                    SOCInAddr   addr;
                    addr.addr = (u32) hotEnt->addrList;
                    OS_TPrintf("Resolved IP=%s\n", SOC_InetNtoA(addr));
                }
                else
                {
                    OS_TPrintf("NO RESOLVER or NO ENTRY\n");
                }
            }
        }
        else
        {
// Under the current WCM spec, the ability to detect a link break
// occurs several seconds after the actual link is broken,
// so it is possible that the DHCP might get a timeout first.
// As a result, DHCP failures can be seen as resulting from link breaks
// and when re-connecting, it will be safer to begin by establishing the link.
            OS_TPrintf("NO DHCP SERVER or NO LINK....\n");

            switch (SOC_GetConfigError(NULL))
            {
            case SOC_IP_ERR_DHCP_TIMEOUT:
                OS_TPrintf("SOC_IP_ERR_DHCP_TIMEOUT\n");
                break;

            case SOC_IP_ERR_LINK_DOWN:
                OS_TPrintf("SOC_IP_ERR_LINK_DOWN\n");
                break;

            default:
                OS_TPrintf("Default???\n");
                break;
            }
        }

        OS_TPrintf("LINK DOWN....\n");
        (void)SOC_Cleanup();
        TerminateWcmControl();
        while (WCM_GetPhase() != WCM_PHASE_NULL)
        {
// when the priority for the main thread is higher than that of the WCM thread
// you need to clearly vacate the process
            OS_Sleep(100);
        }
    }

    OS_TPrintf("Terminate....\n");
    OS_Terminate();
}

/*---------------------------------------------------------------------------*
heap-specific
 *---------------------------------------------------------------------------*/
static void Heap_Setup(void)
{
    void*           nstart;
    OSHeapHandle    handle;

    nstart = OS_InitAlloc(OS_ARENA_MAIN, OS_GetMainArenaLo(), OS_GetMainArenaHi(), 1);
    OS_SetMainArenaLo(nstart);
    handle = OS_CreateHeap(OS_ARENA_MAIN, OS_GetMainArenaLo(), OS_GetMainArenaHi());
    (void)OS_SetCurrentHeap(OS_ARENA_MAIN, handle);
}

static void* myAlloc(u32 name, s32 size)
{
#pragma unused(name)
    OSIntrMode  enable = OS_DisableInterrupts();
    void*       ptr = OS_Alloc((u32) size);
    (void)OS_RestoreInterrupts(enable);
    return ptr;
}

static void myFree(u32 name, void* ptr, s32 size)
{
#pragma unused(name, size)
    OSIntrMode  enable = OS_DisableInterrupts();
    OS_Free(ptr);
    (void)OS_RestoreInterrupts(enable);
    return;
}

/*====== End of dhcp_monkey.c ======*/
