/*---------------------------------------------------------------------------*
Project: NitroWiFi - SOC - demos - httpclient
File: ssl.h

Copyright 2005.2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: ssl.h,v $
Revision 1.2  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.1  12/22/2005 04:34:06  adachi_hiroaki
replaced HTTP client demo with SOC version

$NoKeywords$
 *---------------------------------------------------------------------------*/
#ifndef HTTPCLIENT_SSL_H_
#define HTTPCLIENT_SSL_H_

#ifdef __cplusplus

extern "C" {
#endif

#include <nitroWiFi.h>

/*===========================================================================*/

/*---------------------------------------------------------------------------*
constant definitions
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
structure definitions
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
function prototype definitions
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
Name: SslInit

Description: SSL initialization

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    SslInit(void);

/*---------------------------------------------------------------------------*
Name: SslSetDefaultConnection

Description: registers the default SOCLSslConnection structure

Arguments: psslCon		pointer to the registering SOCLSslConnection

Returns: None.
 *---------------------------------------------------------------------------*/
void    SslSetDefaultConncetion(SOCSslConnection* psslCon);

/*---------------------------------------------------------------------------*
Name: SslGetDefaultConnection

Description: returns a pointer to the default SOCLSslConnection registered in ssl.c

Arguments: None.

Returns: a pointer to the SOCLSslConnection registered in ssl.c
returns NULL prior to InitSSL
 *---------------------------------------------------------------------------*/
SOCLSslConnection*  SslGetDefaultConnection();

/*---------------------------------------------------------------------------*
Name: SslSetDefaultAuthCallback

Description: sets a callback for SSL authentication

Arguments: auth_callback	pointer to the callback function

Returns:		None.
 *---------------------------------------------------------------------------*/
void    SslSetDefaultAuthCallback(int (*auth_callback) (int, struct _CPSSslConnection*, int));

/*---------------------------------------------------------------------------*
Name: SslAuthCallback

Description: SSL server authentication callback

Arguments: con           SSL connection data
level         level

Returns: None.
 *---------------------------------------------------------------------------*/
int     SslAuthCallback(int result, SOCSslConnection* con, int level);

/*---------------------------------------------------------------------------*
Name: SslSetDefaultCA

Description: sets the SSL certificate

Arguments: ca_list		certificate list
builtins.	the number of certificates

Returns:		None.
 *---------------------------------------------------------------------------*/
void    SslSetDefaultCA(SOCCaInfo ** ca_list, int builtins);

/*---------------------------------------------------------------------------*
Name: SslCreateConnection

Description: sets the SSL certificate

Arguments: servername	target server name that matches the common name

Returns:		pointer to the newly allocated SslCreateConnection structure
 *---------------------------------------------------------------------------*/
SOCLSslConnection*  SslCreateConnection(const char* servername);

/*---------------------------------------------------------------------------*
Name: SslDestroyConnection

Description: destroys the SslCreateConnection structure associated with the socket

Arguments: socket		the active SSL socket descriptor

Returns:		pointer to the newly allocated SslCreateConnection structure
 *---------------------------------------------------------------------------*/
void    SslDestroyConnection(SOCSslConnection* con);

void    SslPrintCA(const SOCSslConnection* con);

/*===========================================================================*/
#ifdef __cplusplus

}       /* extern "C" */
#endif /* __cplusplus */
#endif /* HTTPCLIENT_SSL_H_ */
