/*---------------------------------------------------------------------------*
Project: NitroWiFi - SOC - demos - httpclient
File: main.c

Copyright 2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: main.c,v $
Revision 1.6  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.5  01/16/2006 01:40:58  kitase_hirotake
Edited sitedefs for publication

Revision 1.4  01/12/2006 00:10:21  kitase_hirotake
2005->2006 Edited

Revision 1.3  01/10/2006 04:18:39  kitase_hirotake
Changed features to samples

Revision 1.2  01/06/2006 00:44:47  kitase_hirotake
Version compatible with PAD input
  
Revision 1.1  12/22/2005 04:34:06  adachi_hiroaki
replaced HTTP client demo with SOC version

$NoKeywords$
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
This is a sample program that runs as an http client.
It connects to the server by entering the URL with the PAD.

How to enter characters with the PAD:
1. Enter text using the PAD.
Switch between DIRECT_MODE and INDERECT_MODE by pressing the L button.
Text input is done in INDIRECT_MODE.
2. Change the characters being selected with the up and down keys.
Character types can be switched between numbers, upper-case, lower-case, and symbols with the R button.
Confirm the current character and move to the next character input with the A button.
You can highlight and change characters entered up to this point with the left and right keys.
3. Delete the selected character with the B button.
4. Send the string entered to this point with the X button.
    
 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include <nitroWiFi.h>
#include <string.h>
#include <sitedefs.h>

#include "padkeyboard.h"
#include "output.h"
#include "screen.h"

#include "netconnect.h"
#include "http.h"
#include "ssl.h"

#define SITEDEFS_APCLASS    "WiFi.LAN.1.AP.1"

#define URL_LENGTH          64

static void VBlankIntr(void)
{
    OS_SetIrqCheckFlag(OS_IE_V_BLANK);
}

void NitroMain()
{
    char    url[URL_LENGTH];

    /* Calls the initialization function and the connect function. */
    NcGlobalInit();
    NcStart(SITEDEFS_APCLASS);

    /* Initializes the screen */
    InitScreen();

    FlushScreen();
    FlushDebugOutput();

    /* empty call for getting key input data (strategy for pressing A button in IPL) */
    (void)PAD_Read();

    /* Initializes the keyboard. */
    InitKeyboardThread();
    KeyboardValid();

/* Connection to the HTTP server            *
* Input the target URL with the Pad */
    DebugOutput("http_server_ip>");

    /* The initial URL value is the management screen for the router. */
    MI_CpuClear8(url, URL_LENGTH);
    {
        SOCInAddr   addr;
        SOC_GetGateway(NULL, (u8*) &addr);
        (void)OS_SNPrintf(url, sizeof(url), "http://%s/", SOC_InetNtoA(addr));
    }

    while (1)
    {
        PadKeySetString(url);

        /* Enter the URL. */
        PadKeyGetInputString(url, URL_LENGTH);

        {
            static char         pageBuffer[1024 * 20];
            int         socket = -1;
            SOCSslConnection*   sslCon;

            SslInit();
            socket = HttpConnect(url, &sslCon);

            OS_TPrintf("URL: %s\n", url);

            if (socket < 0)
            {
                OS_Panic("failed to connect.");
            }

            if (HttpGet(socket, url, NULL, NULL, 0, pageBuffer, sizeof(pageBuffer)) < 0)
            {
                OS_Panic("failed to download\n");
            }

            OS_PutString(pageBuffer);
            (void)HttpClose(socket, sslCon);
        }
    }

    NcFinish();
    OS_Terminate();
}
