/*---------------------------------------------------------------------------*
Project: NitroWiFi - SOC - demos
File: main.c

Copyright 2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: main.c,v $
Revision 1.4  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.3  01/16/2006 01:41:12  kitase_hirotake
Edited sitedefs for publication

Revision 1.2  01/06/2006 04:31:38  kitase_hirotake
Increased communications speed.

Revision 1.1  01/06/2006 00:43:29  kitase_hirotake
Added httpserver

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include <nitroWiFi.h>
#include <nitro/fs.h>

#include "netconnect.h"

#include "server_thread.h"

#define SITEDEFS_APCLASS    "WiFi.LAN.1.AP.1"

static const u32    default_dma_no = MI_DMA_MAX_NUM;

OSMessage       mesgBuffer[10];
OSMessageQueue      mesgQueue;

/*---------------------------------------------------------------------------*
Sends more data than the communications of normal games,
so the memory usage parameters in the socket are overridden.
 *---------------------------------------------------------------------------*/

// TCP
SDK_WEAK_SYMBOL SOCLSocketParam SOCLSocketParamTCP =
{
    SOCL_FLAGMODE_TCP,                      // TCP
    SOCL_FLAGBLOCK_BLOCK,                   // BLOCK
    {                                       // cps
        0,                                  //SOCL_TCP_SOCKET_CPS_RCVBUF_SIZE,       // rcvbuf_size

//****Set in SOCL_Startup()
        0,                                  //SOCL_TCP_SOCKET_CPS_RCVBUF_CONSUME_MIN,    // rcvbuf_consume_min

//****Set in SOCL_Startup()
        SOCL_TCP_SOCKET_CPS_SNDBUF_SIZE,    // sndbuf_size
        SOCL_TCP_SOCKET_CPS_LINBUF_SIZE,    // linbuf_size
        SOCL_TCP_SOCKET_CPS_OUTBUF_SIZE,    // outbuf_size
        3029,   // The send throughput optimization switch is configured by default.
        0,      // udpbuf_size
    },

    {           // recv_thread
        SOCL_TCP_SOCKET_RECV_THREAD_STACK_SIZE, // stack_size
        SOCL_TCP_SOCKET_RECV_THREAD_PRIORITY,   // priority
        SOCL_TCP_SOCKET_RECV_THREAD_QUEUE_MAX,  // queue_max
    },

    {   // send_thread
        SOCL_TCP_SOCKET_SEND_THREAD_STACK_SIZE, // stack_size
        SOCL_TCP_SOCKET_SEND_THREAD_PRIORITY,   // priority
        SOCL_TCP_SOCKET_SEND_THREAD_QUEUE_MAX,  // queue_max
    },
};

static void VBlankIntr(void)
{
    OSThread*   t;

    OS_SetIrqCheckFlag(OS_IE_V_BLANK);

//OS_CheckStack(OS_GetCurrentThread());
    for (t = OS_GetThreadList(); t != NULL; t = OS_GetNextThread(t))
    {
        OS_CheckStack(t);
    }
}

static void RecursiveEnumDir(int tab, FSDirEntry* pe)
{
    FSFile  d;
    FS_InitFile(&d);

/* in the recursive function, it costs much stack.
so we use one FSDirEntry like this. */
    OS_Printf("%*s%s/\n", tab, "", pe->name);
    if (FS_SeekDir(&d, &pe->dir_id))
    {
        tab += 4;
        while (FS_ReadDir(&d, pe))
        {
            if (pe->is_directory)
                RecursiveEnumDir(tab, pe);
            else
                OS_Printf("%*s%s\n", tab, "", pe->name);
        }
    }
}

/* Initializes memory allocation system in the main memory arena. */
static void InitializeAllocateSystem(void)
{
    void*           tempLo;
    OSHeapHandle    hh;

    /* Based on the premise that OS_Init is already called */
    tempLo = OS_InitAlloc(OS_ARENA_MAIN, OS_GetMainArenaLo(), OS_GetMainArenaHi(), 1);
    OS_SetArenaLo(OS_ARENA_MAIN, tempLo);
    hh = OS_CreateHeap(OS_ARENA_MAIN, OS_GetMainArenaLo(), OS_GetMainArenaHi());
    if (hh < 0)
    {
        OS_Panic("ARM9: Fail to create heap...\n");
    }

    hh = OS_SetCurrentHeap(OS_ARENA_MAIN, hh);
}

static void SetHostSetting(SOSockAddrIn* sa_in, const char* addr, u16 port)
{
    SOHostEnt*  host_ent;

    /* Configures sockaddr_in. */
    sa_in->len = sizeof(*sa_in);
    sa_in->family = SOC_AF_INET;
    if (port > 0)
    {
        sa_in->port = SOC_HtoNs(port);
    }

    if (addr && (host_ent = SOC_GetHostByName(addr)) != 0)
    {
        MI_CpuCopy8(host_ent->addrList[0], &sa_in->addr, SOC_ALEN);
    }
}

/*---------------------------------------------------------------------------*
This is a sample program that runs as an http server.
The only request supported is GET METHOD.
 *---------------------------------------------------------------------------*/
void NitroMain(void)
{
    int server_socket;
    int result;
    int i;
    static SOSockAddrIn server_addr;
    static SOSockAddrIn client_addr;

    OS_Init();
    OS_InitThread();

    OS_InitMessageQueue(&mesgQueue, &mesgBuffer[0], 10);

    /* Calls the shared initialization function. */
    NcGlobalInit();
    NcStart(SITEDEFS_APCLASS);

    /* Specifies V blank */
    (void)OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrqMask(OS_IE_FIFO_RECV);
    (void)OS_EnableIrq();
    (void)OS_EnableInterrupts();
    (void)GX_VBlankIntr(TRUE);

    FS_Init(default_dma_no);

    {
        u32     need_size = FS_GetTableSize();
        void*   p_table = OS_Alloc(need_size);
        SDK_ASSERT(p_table != NULL);
        (void)FS_LoadTable(p_table, need_size);
    }

    /* Starts the server operation. */
    SetHostSetting(&server_addr, "0.0.0.0" /*ANY*/, 80);

    OS_TPrintf("Create....");
    server_socket = SOC_Socket(SOC_PF_INET, SOC_SOCK_STREAM, 0);
    OS_TPrintf("socket=0x%08lx\n", server_socket);
    OS_TPrintf("Bind....");
    result = SOC_Bind(server_socket, &server_addr);
    OS_TPrintf("Listen....");
    result = SOC_Listen(server_socket, 1);
    OS_TPrintf("Accept....\n");

    /* Performs multiple receives using thread. */

    /* Calls OS_InitThread() if it has not been called already. */
    if (!OS_IsThreadAvailable())
    {
        OS_InitThread();
    }

    /* 10 communications threads are made beforehand. */
    for (i = 0; i < 10; i++)
    {
        CreateServerThread();
    }

    /* Accepts multiple clients. */
    while (1)
    {
        result = SOC_Accept(server_socket, &client_addr);
        if (result > 0)
        {
            /* Sends the socket to the message queue and leaves communications to the thread once the connection has been accepted. */
            (void)OS_SendMessage(&mesgQueue, (OSMessage) (&result), OS_MESSAGE_BLOCK);
        }
    }

    /* It doesn't reach this far. */
    NcFinish();
    OS_TPrintf("Terminate....\n");
    OS_Terminate();
}
