/*---------------------------------------------------------------------------*
Project: NitroWiFi - demo - SOC - shared
File: ftp.h

Copyright 2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: ftp.h,v $
Revision 1.2  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.1  01/12/2006 02:16:14  okubata_ryoma
ftp.h additions

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#if !defined(NITROWIFI_SOC_SHARED_FTP_H_)
#define NITROWIFI_SOC_SHARED_FTP_H_

#include <nitro.h>
#include <nitroWiFi.h>
#include <stdlib.h>

#include "padkeyboard.h"

#ifdef __cplusplus

extern "C"
{
#endif

/*****************************************************************************/

/* constant */
#define FTPi_BUFFERSIZE_COMMAND 256
#define FTPi_BUFFERSIZE_DATA    1440

#define FTPi_BUFFERSIZE_DOWN    1440
#define FTPi_BUFFERSIZE_UP      14600

#define FTPi_RETURN_ERROR(x) \
    { \
        int CODE = (x); \
        if (CODE < 0) \
            return CODE; \
    }

#define UPLOAD      13
#define DOWNLOAD    14
#define NLST        15
#define QUIT        16
#define ASCII       17
#define BINARY      18
#define LS          19
#define HELP        20

#define FILEDATA16  16
#define FILEDATA256 256

/*****************************************************************************/

/* function */

/*---------------------------------------------------------------------------*
Name: FTP_Open

Description: Connects to the FTP server.

Arguments: http_addr     : Connection target address
http_port     : Connection target port

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_Open(const char* http_addr, u16 http_port);

/*---------------------------------------------------------------------------*
Name: FTP_Login

Description: Issues the USER and PASS commands and logs-in to the FTP server.

Arguments: user_name     : Connection target account
password: Connection target password
                

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_Login(const char* user_name, const char* password);

/*---------------------------------------------------------------------------*
Name: FTP_Passive

Description: Establishes a data connection in Passive mode.

Arguments: filetype File transmission mode TRUE  : BINARY mode
FALSE : ASCII mode

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_Passive(BOOL filetype);

/*---------------------------------------------------------------------------*
Name: FTP_OnlyStor

Description: Only sends the STOR command

Arguments: filename: Filename for upload.

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_OnlyStor(const char* filename);

/*---------------------------------------------------------------------------*
Name: FTP_OnlySendData

Description: Asynchronously runs SOC_Send once.

Arguments: size   : Size to send.

Returns: Size uploaded with this send.
 *---------------------------------------------------------------------------*/
int     FTP_OnlySendData(int size);

/*---------------------------------------------------------------------------*
Name: FTP_OnlyRetr

Description: Only sends the RETR command

Arguments: filename: Filename for download.

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_OnlyRetr(const char* filename);

/*---------------------------------------------------------------------------*
Name: FTP_OnlyRecvData

Description: Asynchronously runs SOC_Recv once.

Arguments: filesize    : Data transmission size.
recv_filename    : When using the NLST command, enter the received file name.

Returns: Size downloaded with this recv.
 *---------------------------------------------------------------------------*/
int     FTP_OnlyRecvData(int filesize, char recv_filename[FILEDATA256]);

/*---------------------------------------------------------------------------*
Name: FTP_CloseDataSocket

Description: Closes FTPi_DataSocket and disconnects.

Arguments: none.

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_CloseDataSocket(void);

/*---------------------------------------------------------------------------*
Name: FTP_OnlyAbor

Description: Only sends the ABOR command (doesn't even receive a corresponding message).

Arguments: none.

Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_OnlyAbor();

/*---------------------------------------------------------------------------*
Name: FTP_OnlySendCommand

Description: Performs FTPi_Print.

Arguments: fmt   : String to write.


Returns: Values >= 0 indicate success.
 *---------------------------------------------------------------------------*/
int     FTP_OnlySendCommand(const char* fmt, ...);

/*---------------------------------------------------------------------------*
Name: FTP_OnlyRecvCommand

Description: Runs FTPi_Readline

Arguments: none.

Returns: Values >= 0 indicates success.
 *---------------------------------------------------------------------------*/
int     FTP_OnlyRecvCommand(void);

/*---------------------------------------------------------------------------*
Name: FTP_FileSize

Description: Gets the size of the specified file name.

Arguments: filename: File name to examine.

Returns: Size of the file.
 *---------------------------------------------------------------------------*/
int     FTP_FileSize(const char* filename);

/*---------------------------------------------------------------------------*
Name: FTP_Quit

Description: Quits the FTP connection.

Arguments: None

Returns:      >= 0 indicates success.
 *---------------------------------------------------------------------------*/
int     FTP_Quit(void);

/*---------------------------------------------------------------------------*
Name: FTP_Upload

Description: Performs file upload.

Arguments: pfiletype    : File transmission mode.
pfilesize    : Data transmission size.
pfilename    : File name

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_Upload(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_Download

Description: Performs file download.

Arguments: pfiletype    : File transmission mode.
pfilesize    : Data transmission size.
pfilename    : File name

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_Download(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_Nlst

Description: Displays a list of files.

Arguments: pfiletype    : File transmission mode.
pfilesize    : Data transmission size (unused).
pfilename    : Filename (unused).

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_Nlst(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_Logout

Description: Logout

Arguments: pfiletype    : File transmission mode (unused).
pfilesize    : Data transmission size (unused).
pfilename    : Filename (unused).

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_Logout(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_ASCMode

Description: Changed the file transmission mode to ASCII mode.

Arguments: pfiletype    : File transmission mode.
pfilesize    : Data transmission size (unused).
pfilename    : Filename (unused).

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_ASCMode(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_BINMode

Description: Changed the file transmission mode to BINARY mode.

Arguments: pfiletype    : File transmission mode.
pfilesize    : Data transmission size (unused).
pfilename    : Filename (unused).

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_BINMode(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_Ls

Description: Displays a list of files.

Arguments: pfiletype    : File transmission mode (unused).
pfilesize    : Data transmission size (unused).
pfilename    : Filename (unused).

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_Ls(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_Help

Description: Displays help.

Arguments: pfiletype    : File transmission mode (unused).
pfilesize    : Data transmission size (unused).
pfilename    : Filename (unused).

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_Help(void* pfiletype, void* pfilesize, void* pfilename);

/*---------------------------------------------------------------------------*
Name: FTP_LIST

Description: Displays a list of files.

Arguments: pfiletype    : File transmission mode.
pfilename    : Filenames used only with the LIST command.
download_filedata    : Data from the file to be downloaded.
option               FALSE : NLST -1
TRUE  : LIST filename

Returns: None.
 *---------------------------------------------------------------------------*/
void    FTP_LIST(void* pfiletype, void* pfilename, char download_filename[FILEDATA256], int option);

/*---------------------------------------------------------------------------*
Name: RecursiveEnumDirEx

Description: Displays and saves a list of the filenames in the directory.

Arguments: tab                : Width of the subscript.
pe                 : The address of the target directory and the FSDirEntry structure.
output_main        : If TRUE: Displays the directories and filenames below that directory.
If FALSE: Saves the filenames in that directory.
upload_filename    : Destination of the filename.

Returns: The number of files displayed or saved.
 *---------------------------------------------------------------------------*/
int     RecursiveEnumDirEx(int tab, FSDirEntry* pe, BOOL output_main, char upload_filename[FILEDATA256]);

#ifdef __cplusplus

} /* extern "C" */
#endif

#endif /* NITROWIFI_SOC_SHARED_FTP_H_ */
