/*---------------------------------------------------------------------------*
Project: NitroWiFi - SOC - shared
File: padkeyboard.h

Copyright 2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: padkeyboard.h,v $
Revision 1.3  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.2  01/10/2006 04:21:17  kitase_hirotake
Added PadKeySetString and PadKeyGetInputString

Revision 1.1  01/06/2006 00:23:35  kitase_hirotake
Initial Upload

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#if !defined(NITROWIFI_SOC_SHARED_PAD_KEYBOARD_H_)
#define NITROWIFI_SOC_SHARED_PAD_KEYBOARD_H_

#include <nitro.h>

#include <stdlib.h>

#include "output.h"
#include "screen.h"

#ifdef __cplusplus

extern "C"
{
#endif

/*****************************************************************************/

/* constant */
#define INPUT_MAX   256

/* Constants used by the keyboard */
#define KEY_UP              (-1)
#define KEY_DOWN            (-2)
#define KEY_LEFT            (-3)
#define KEY_RIGHT           (-4)
#define KEY_BACKSPACE       (-5)
#define KEY_SPACE           (-6)
#define KEY_RETURN          (-7)
#define KEY_DONT_GET        (-128)

#define DIRECT_MODE         1
#define INDIRECT_MODE       2

#define ARABIC_NUMERALS     1
#define CAPITAL_LETTER      2
#define SMALL_LETTER        3
#define CHARACTOR_SYMBOL    4
#define MAX_CHARACTOR_MODE  4

/* Macro for checking whether the data acquired with PadKeyGet* is in Indirect Mode */
#define IndirectCheck(charactor)    (charactor > KEY_UP ? 1 : 0)

/*---------------------------------------------------------------------------*
Name: KeyboardValid, KeyboardInvalid

Description: Switches keyboard input between valid and invalid

Arguments: None.

Returns: Nothing
 *---------------------------------------------------------------------------*/
void    KeyboardValid();
void    KeyboardInvalid();

/*---------------------------------------------------------------------------*
Name: PadKeySetString

Description: Sets the specified string to the current input

Arguments: string - String being set.

Returns: Nothing
 *---------------------------------------------------------------------------*/
void    PadKeySetString(char string[]);

/*---------------------------------------------------------------------------*
Name: PadKeyGetInputString

Description: Causes and receives string input.

Arguments: string - String being set.

Returns: Nothing
 *---------------------------------------------------------------------------*/
void    PadKeyGetInputString(char string[], int size);

/*---------------------------------------------------------------------------*
Name: PadKeyGetcharAsync

Description: Gets one character from pad input data (asynchronous).

Arguments: None.

Returns: Character data.
 *---------------------------------------------------------------------------*/
char    PadKeyGetcharAsync(void);

/*---------------------------------------------------------------------------*
Name: PadKeyGetchar

Description: Gets one character from pad input data (synchronous).

Arguments: None.

Returns: Character data.
 *---------------------------------------------------------------------------*/
char    PadKeyGetchar(void);

/*---------------------------------------------------------------------------*
Name: WakeupKeyboardThread

Description: Wakes the keyboard thread.

Arguments: None.

Returns: Nothing
 *---------------------------------------------------------------------------*/
void    WakeupKeyboardThread(void);

/*---------------------------------------------------------------------------*
Name: InitKeyboardThread

Description: Creates a keyboard thread.

Arguments: None.

Returns: Nothing
 *---------------------------------------------------------------------------*/
void    InitKeyboardThread(void);

#ifdef __cplusplus

} /* extern "C" */
#endif

#endif /* NITROWIFI_SOC_SHARED_PAD_KEYBOARD_H_ */
