/*---------------------------------------------------------------------------*
Project: NitroWiFi - SOC - shared
File: screen.h

Copyright 2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: screen.h,v $
Revision 1.2  03/10/2006 09:22:43  kitase_hirotake
  INDENT SOURCE

Revision 1.1  01/06/2006 00:23:35  kitase_hirotake
Initial Upload

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#if !defined(NITROWIFI_SOC_SHARED_SCREEN_H_)
#define NITROWIFI_SOC_SHARED_SCREEN_H_

#include <nitro.h>

#ifdef __cplusplus

extern "C"
{
#endif

/*****************************************************************************/

/* constant */
extern const u32    d_CharData[8 * 256];
extern const u32    d_PaletteData[8 * 16];

#define COLOR_WHITE             0
#define COLOR_RED               1
#define COLOR_GREEN             2
#define COLOR_BLUE              3
#define COLOR_YELLOW            4
#define COLOR_PURPLE            5
#define COLOR_LIGHT_BLUE        6
#define COLOR_DARK_RED          7
#define COLOR_DARK_GREEN        8
#define COLOR_DARK_BLUE         9
#define COLOR_DARK_YELLOW       10
#define COLOR_DARK_PURPLE       11
#define COLOR_DARK_LIGHT_BLUE   12
#define COLOR_GRAY              13
#define COLOR_DARK_GRAY         14
#define COLOR_DARK_BLACK        15

/*****************************************************************************/

/* function */

/*---------------------------------------------------------------------------*
Name: InitScreen

Description: Initializes the display operation.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    InitScreen(void);

/*---------------------------------------------------------------------------*
Name: ClearScreen

Description: Clears the text screen

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    ClearScreen(void);

/*---------------------------------------------------------------------------*
Name: PadkeyClearScreen

Description: Clears the portion of the text screen used by Padkey.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    PadkeyClearScreen(void);

/*---------------------------------------------------------------------------*
Name: PrintString

Description: Displays a string on the text screen.

Arguments: x             X coordinates in grid units
y             Y coordinates in grid units
palette       Palette index
format        Format string

Returns: None.
 *---------------------------------------------------------------------------*/
void    PrintString(int x, int y, int palette, const char* format, ...);

/*---------------------------------------------------------------------------*
Name: PrintDebugString

Description: Displays the debug string in the sub-screen's text screen.

Arguments: s                Output string.

Returns: None.
 *---------------------------------------------------------------------------*/
void    PrintDebugString(const char* s, ...);

/*---------------------------------------------------------------------------*
Name: FlushScreen

Description: Here, the text screen is reflected in VRAM.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void    FlushScreen(void);

/*---------------------------------------------------------------------------*
Name: GetFlickerColor

Description: Gets the text color for flashing.

Arguments: None.

Returns: int     -   Palette number
 *---------------------------------------------------------------------------*/
int     GetFlickerColor(void);

/*---------------------------------------------------------------------------*
Name: ShowNavigater

Description: Displays a message introducing the operator on the bottom line of the text screen.
                 It flashes on and off by being called each frame.

Arguments: text    -   The text of the displayed message.

Returns: None.
 *---------------------------------------------------------------------------*/
void    ShowNavigater(const char* text);

/*---------------------------------------------------------------------------*
Name: PrintStep

Description: Displays the procedural string in the specified location of the text screen.
Inserts line-breaks appropriately depending on the number of characters and line width.

Arguments: x       -   x coordinate of the display origin point
y       -   y coordinate of the display origin point
palette -   Display palette color (COLOR_*)
step    -   Current step number
comment -   Comment string that displays the test procedure.

Returns: The number of lines needed to display.
 *---------------------------------------------------------------------------*/
int     PrintStep(int x, int y, int palette, int step, const char* comment);

#ifdef __cplusplus

} /* extern "C" */
#endif

#endif /* NITROWIFI_SOC_SHARED_SCREEN_H_ */
