/*---------------------------------------------------------------------------*
Project: NitroWiFi - SSL - demos - ssl-1
File: wcm_control.c

Copyright 2005.2006 Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.

$Log: wcm_control.c,v $
Revision 1.3  03/10/2006 09:22:42  kitase_hirotake
  INDENT SOURCE

Revision 1.2  2005/11/02 02:41:33  terui
Fix comment.

Revision 1.1  07/21/2005 11:47:03  seiki_masashi
moved ssl to ssl-1

Revision 1.1  07/18/2005 08:10:26  seiki_masashi
added ssl-1 demo

Revision 1.1  2005/07/14 10:15:49  jey-s
*** empty log message ***

Revision 1.2  07/12/2005 04:17:50  terui
improved fixed WEP settings display
improve switching for setting for verification method

Revision 1.1  07/11/2005 12:07:09  terui
new additions

$NoKeywords: $
 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include <nitro/os.h>
#include <nitroWiFi.h>
#include "wcm_control.h"
#include "screen.h"

/*---------------------------------------------------------------------------*
AP data

NOTICE: Edit this definition to match the AP to which you will connect.
Furthermore, this sample will automatically connect to an AP indicating
BSSID, ESSID, the 802.11 verification method and the WEP encryption settings herein.
 *---------------------------------------------------------------------------*/
static const u8*        gWcmBssid = WCM_BSSID_ANY;
static const char       gWcmEssid[] = { "000D0B14ABD2" };
static const u32        gWcmAuthOption = WCM_OPTION_AUTH_OPENSYSTEM;

#define LOCAL_WCM_WEP_MODE  WCM_WEPMODE_NONE

static const WCMWepDesc gWcmWepDesc =
{
    LOCAL_WCM_WEP_MODE ,                        // encryption mode
    0,                                          // WEP key used during send (0 to 3)
#if ( LOCAL_WCM_WEP_MODE == WCM_WEPMODE_40 )
    {
        0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    }
#elif ( LOCAL_WCM_WEP_MODE == WCM_WEPMODE_104 )
    {
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    }
#elif ( LOCAL_WCM_WEP_MODE == WCM_WEPMODE_128 )
    {
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00
    }
#else /* ( LOCAL_WCM_WEP_MODE == WCM_WEPMODE_NONE ) */
    {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
                    }
#endif
};

/*---------------------------------------------------------------------------*
constant definitions
 *---------------------------------------------------------------------------*/
#define LOCAL_WCM_THREAD_STACK_SIZE     1024    // Allocates a large stack (can be changed)
#define LOCAL_WCM_THREAD_PRIORITY       24      // a low priority is sufficient (can be changed)
#define LOCAL_WCM_DMA_NO                3       // DMA number used in wireless functionality (can be changed)
#define LOCAL_WCM_APLIST_COUNT_MAX      1       // the number of AP data lists to be managed within the WCM library
#define LOCAL_WCM_APLIST_BUF_SIZE       ((LOCAL_WCM_APLIST_COUNT_MAX * WCM_APLIST_BLOCK_SIZE) + 12)
#define LOCAL_WCM_THREAD_MSGQ_CAPACITY  2       // the number of levels in the message queue
#define LOCAL_WCM_MSG_TERMINATE         ((void*)0)
#define LOCAL_WCM_MSG_FOUND_AP          ((void*)1)
#define LOCAL_WCM_MSG_DISCONNECTED      ((void*)2)

// string constant for displaying the WCM library operations log
static const char*      gWcmResultText[8] =
{
    { "SUCCESS" } ,         { "FAILURE" } ,
    { "PROGRESS" } ,        { "ACCEPT" } ,
    { "REJECT" } ,          { "WMDISABLE" } ,
    { "NOT_ENOUGH_MEM" } ,  { "FATAL_ERROR" }
};
static const char*      gWcmNotifyText[10] =
{
    { "COMMON" } ,          { "STARTUP" } ,
    { "CLEANUP" } ,         { "BEGIN_SEARCH" } ,
    { "END_SEARCH" } ,      { "CONNECT" } ,
    { "DISCONNECT" } ,      { "FOUND_AP" } ,
    { "SEARCH_AROUND" } ,   { "TERMINATE" }
};

/*---------------------------------------------------------------------------*
internal variable definitions
 *---------------------------------------------------------------------------*/

// for thread controls
static OSThread         gWcmThread;
static u32          gWcmStack[LOCAL_WCM_THREAD_STACK_SIZE / sizeof( u32 ) ];
static OSMessageQueue   gWcmMessageQ;
static OSMessage        gWcmMsgArray[LOCAL_WCM_THREAD_MSGQ_CAPACITY];
static OSThreadQueue    gWcmThreadQ;

// for WCM library controls
static volatile BOOL    gWcmTerminateFlag;
static vs32         gWcmResult;
static void*        gWcmBuf = NULL;
static WCMConfig    gWcmConfig;
static WMBssDesc gWcmBssDesc    ATTRIBUTE_ALIGN(4);

/*---------------------------------------------------------------------------*
internal function prototype
 *---------------------------------------------------------------------------*/
static void ControlThread(void* arg);
static void FreezeThread(void);
static s32  CallFunction(s16 notifyId);
static void ControlCallback(WCMNotice* arg);

/*---------------------------------------------------------------------------*
Name: InitWcmControl

Description: Initializes internal variables and launches the WCM control thread.
Nothing is performed if the thread has already been launched.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void InitWcmControl(void)
{
// Is the thread variable initialized even once?
    if (gWcmThread.id != 0)
    {
// Is the thread in an ended state?
        if (OS_IsThreadTerminated(&gWcmThread) == FALSE)
        {
            return;
        }
    }

// initializes the message queue
    OS_InitMessageQueue(&gWcmMessageQ, gWcmMsgArray, LOCAL_WCM_THREAD_MSGQ_CAPACITY);

// initializes the thread queue
    OS_InitThreadQueue(&gWcmThreadQ);

// initializes the variables
    MI_CpuClearFast(&gWcmBssDesc, sizeof(WMBssDesc));

// creates a new thread using the thread variables
    OS_CreateThread(&gWcmThread, ControlThread, NULL, (void*)(gWcmStack + LOCAL_WCM_THREAD_STACK_SIZE),
                    LOCAL_WCM_THREAD_STACK_SIZE, LOCAL_WCM_THREAD_PRIORITY);

// launches the created thread
    gWcmTerminateFlag = FALSE;
    OS_WakeupThreadDirect(&gWcmThread);
}

/*---------------------------------------------------------------------------*
Name: TerminateWcmControl

Description: Ends the WCM control thread.
Nothing is performed if the thread has not been launched.
NOTICE: As this function only serves as a trigger to end the thread,
be aware that it will return without waiting for the thread itself to end.
If you want to wait for the thread itself to end, you will need to call
OS_JoinThread after calling OS_SendMessage.
In addition, if the WCM_GetPhase is set to return WCM_PHASE_NULL,
the thread has ended.

Arguments: None.

Returns: None.
 *---------------------------------------------------------------------------*/
void TerminateWcmControl(void)
{
    if (gWcmThread.id != 0)
    {
        if (OS_IsThreadTerminated(&gWcmThread) == FALSE)
        {
            if (gWcmTerminateFlag == FALSE)
            {
                gWcmTerminateFlag = TRUE;
                (void)OS_SendMessage(&gWcmMessageQ, LOCAL_WCM_MSG_TERMINATE, OS_MESSAGE_NOBLOCK);
            }
        }
    }
}

/*---------------------------------------------------------------------------*
Name: ControlThread

Description: Thread performing WCM library state management.
While the gWcmTerminateFlag variable is FALSE attempts will be made to connect to the specified AP
or to maintain that connection if a connection has already been made.
When the TerminateWcmControl function is called, the gWcmTerminateFlag variable
is changed to true, and subsequently the WCM library will end and the thread itself
will attempt to end.
In addition, if a FATAL ERROR occurs within the WCM library,
this thread will internally enter an infinite loop.

Arguments: arg     -   Not used.

Returns: None.
 *---------------------------------------------------------------------------*/
static void ControlThread(void* arg)
{
#pragma unused(arg)

    void*       wcmBuf;
    void*       wcmAplistBuf;
    s32         wcmResult;
    OSMessage   tempMessage;

// blocks off the memory to be used by the WCM library
    wcmBuf = OS_Alloc(WCM_WORK_SIZE + LOCAL_WCM_APLIST_BUF_SIZE);

    if (wcmBuf == NULL)
    {
        /* if the memory is unavailable, subsequent operations will fail, so end */
        return;
    }

    wcmAplistBuf = (void*)((u32) wcmBuf + WCM_WORK_SIZE);

// initializes the WCM library
    wcmResult = WCM_Init(wcmBuf, WCM_WORK_SIZE);
    PrintSubScreen("WCM_Init() -> %s\n", gWcmResultText[wcmResult]);
    if (wcmResult != WCM_RESULT_SUCCESS)
    {
        /*
* If an error is returned here, it is highly possible that use of the library is in error.
* It is possible that other threads have already initialized the WCM library.
         */
        OS_Free(wcmBuf);
        return;
    }

// WCM library operation settings
    gWcmConfig.dmano = LOCAL_WCM_DMA_NO;
    gWcmConfig.pbdbuffer = wcmAplistBuf;
    gWcmConfig.nbdbuffer = LOCAL_WCM_APLIST_BUF_SIZE;
    gWcmConfig.nbdmode = WCM_APLIST_MODE_IGNORE;

// connection start
    while (gWcmTerminateFlag == FALSE)
    {
        switch (WCM_GetPhase())
        {
        case WCM_PHASE_WAIT:
// launch wireless functionality
            wcmResult = CallFunction(WCM_NOTIFY_STARTUP);
            break;

        case WCM_PHASE_IDLE:
            if (gWcmBssDesc.length == 0)
            {
// begin AP auto-search
                WCM_ClearApList();
                wcmResult = CallFunction(WCM_NOTIFY_BEGIN_SEARCH);
            }
            else
            {
// connect to AP
                wcmResult = CallFunction(WCM_NOTIFY_CONNECT);
                if (wcmResult == WCM_RESULT_SUCCESS)
                {
// clear here to re-perform search when attempting to connect next
                    MI_CpuClearFast(&gWcmBssDesc, sizeof(WMBssDesc));
                }
            }
            break;

        case WCM_PHASE_SEARCH:
// wait until an AP is detected
            (void)OS_ReceiveMessage(&gWcmMessageQ, &tempMessage, OS_MESSAGE_BLOCK);

            /*
* Two types of messages can be received: a WCM stop request or an AP detection
notification. If an AP detection notification, the AP detected by this sample is
* the AP to which the connection should be made, and the data is cleared.
             */
            if (tempMessage == LOCAL_WCM_MSG_FOUND_AP)
            {
                (void)WCM_LockApList(WCM_APLIST_LOCK);
                if (WCM_CountApList() == 1)
                {
// As the BSSID is fixed, there should only be one discoverable AP
                    MI_CpuCopyFast(WCM_PointApList(0), &gWcmBssDesc, WCM_APLIST_SIZE);
                }
                (void)WCM_LockApList(WCM_APLIST_UNLOCK);
                wcmResult = CallFunction(WCM_NOTIFY_END_SEARCH);
            }
            break;

        case WCM_PHASE_DCF:
// wait until disconnect notification
            (void)OS_ReceiveMessage(&gWcmMessageQ, &tempMessage, OS_MESSAGE_BLOCK);

            /*
* Two types of messages can be received: a WCM stop request or an AP disconnect
notification. No process is to be performed regardless of the message received.
             */
            break;

        case WCM_PHASE_IRREGULAR:
// a forced stop is the only enabled process during the IRREGULAR phase
            wcmResult = CallFunction(WCM_NOTIFY_TERMINATE);
            break;

        case WCM_PHASE_FATAL_ERROR:
        default:
            FreezeThread();
        }
    }

// end process
    while (gWcmTerminateFlag != FALSE)
    {
        switch (WCM_GetPhase())
        {
        case WCM_PHASE_WAIT:
// escape from the loop
            gWcmTerminateFlag = FALSE;
            break;

        case WCM_PHASE_IDLE:
// stop wireless functionality
            wcmResult = CallFunction(WCM_NOTIFY_CLEANUP);
            break;

        case WCM_PHASE_SEARCH:
// stop AP auto-search
            wcmResult = CallFunction(WCM_NOTIFY_END_SEARCH);
            break;

        case WCM_PHASE_DCF:
// disconnect from the AP
            wcmResult = CallFunction(WCM_NOTIFY_DISCONNECT);
            break;

        case WCM_PHASE_IRREGULAR:
// forced end to wireless functionality
            wcmResult = CallFunction(WCM_NOTIFY_TERMINATE);
            break;

        case WCM_PHASE_FATAL_ERROR:
        default:
            FreezeThread();
        }
    }

// ends the WCM library
    wcmResult = WCM_Finish();
    PrintSubScreen("WCM_Finish() -> %s\n", gWcmResultText[wcmResult]);
    if (wcmResult != WCM_RESULT_SUCCESS)
    {
        /*
* If an error is returned here, it is highly possible that use of the library is in error.
* It is possible that other threads have already ended the WCM library.
         */
        OS_Free(wcmBuf);
        return;
    }

// release the blocked off memory
    OS_Free(wcmBuf);
    return;
}

/*---------------------------------------------------------------------------*
Name: FreezeThread

Description: Called from the thread, to enter an infinite loop and freeze.

Arguments: None.

Returns: There are no returns from this function.
 *---------------------------------------------------------------------------*/
static void FreezeThread(void)
{
    /*
* As there is nothing that can be done for a FATAL ERROR, set to freeze internally.
* Possible causes include fatal memory corruption, an overloading of ARM7
* causing processes to fail or a thread stack overflow.
* The end user must be shown an appropriate error message, but as
* there is no way in software to recover, the use can only restart the device.
     */
    while (TRUE)
    {
        OS_Sleep(10000);
    }
}

/*---------------------------------------------------------------------------*
Name: CallFunction

Description: Calls the WCM asynchronous API and waits until the asynchronous process completes.
Assumed to be called by a thread, be aware that it will wait it
the completion of the asynchronous process is blocked internally.

Arguments: The WCM asynchronous API types are specifies in the WCM_NOTIFY_* format.
The following are the specifiable parameters:
WCM_NOTIFY_STARTUP      : Runs WCM_StartupAsync.
WCM_NOTIFY_CLEANUP      : Runs WCM_CleanupAsync.
WCM_NOTIFY_BEGIN_SEARCH : Runs WCM_SearchAsync (start).
WCM_NOTIFY_END_SEARCH   : Runs WCM_SearchAsync (stop).
WCM_NOTIFY_CONNECT      : Runs WCM_ConnectAsync.
WCM_NOTIFY_DISCONNECT   : Runs WCM_DisconnectAsync.
WCM_NOTIFY_TERMINATE    : Runs WCM_TerminateAsync.

Returns: Returns synchronous and asynchronous process results.
The returned value is one of the following:
WCM_RESULT_SUCCESS      : Process completed normally.
WCM_RESULT_FAILURE      : Process failed midway for some reason.
WCM_RESULT_FATAL_ERROR  : beyond assumed scope of parameters
 *---------------------------------------------------------------------------*/
static s32 CallFunction(s16 notifyId)
{
    s32 wcmResult = WCM_RESULT_FATAL_ERROR;

    while (TRUE)
    {
// delineates the API to be run
        switch (notifyId)
        {
        case WCM_NOTIFY_STARTUP:
            wcmResult = WCM_StartupAsync(&gWcmConfig, ControlCallback);
            PrintSubScreen("WCM_StartupAsync() -> %s\n", gWcmResultText[wcmResult]);
            break;

        case WCM_NOTIFY_CLEANUP:
            wcmResult = WCM_CleanupAsync();
            PrintSubScreen("WCM_CleanupAsync() -> %s\n", gWcmResultText[wcmResult]);
            break;

        case WCM_NOTIFY_BEGIN_SEARCH:
            /****************************************************************/

            /*  place where critical settings are performed for searching for an access point    */
            wcmResult = WCM_SearchAsync((void*)gWcmBssid, (void*)gWcmEssid,
                                        WCM_OPTION_CHANNEL_ALL | WCM_OPTION_SCANTYPE_ACTIVE | WCM_OPTION_ROUNDSCAN_IGNORE);

            /****************************************************************/
            PrintSubScreen("WCM_SearchAsync(S) -> %s\n", gWcmResultText[wcmResult]);
            break;

        case WCM_NOTIFY_END_SEARCH:
            wcmResult = WCM_SearchAsync(NULL, NULL, 0);
            PrintSubScreen("WCM_SearchAsync(E) -> %s\n", gWcmResultText[wcmResult]);
            break;

        case WCM_NOTIFY_CONNECT:
            /****************************************************************/

            /*  place where critical settings are performed for connecting to an access point    */
            wcmResult = WCM_ConnectAsync(&gWcmBssDesc, (void*)(&gWcmWepDesc), WCM_OPTION_POWER_ACTIVE | gWcmAuthOption);

            /****************************************************************/
            PrintSubScreen("WCM_ConnectAsync() -> %s\n", gWcmResultText[wcmResult]);
            break;

        case WCM_NOTIFY_DISCONNECT:
            wcmResult = WCM_DisconnectAsync();
            PrintSubScreen("WCM_DisconnectAsync() -> %s\n", gWcmResultText[wcmResult]);
            break;

        case WCM_NOTIFY_TERMINATE:
            wcmResult = WCM_TerminateAsync();
            PrintSubScreen("WCM_TerminateAsync() -> %s\n", gWcmResultText[wcmResult]);
            break;

        default:
            return wcmResult;
        }

// confirms synchronous process results
        switch (wcmResult)
        {
        case WCM_RESULT_REJECT:
// retries after waiting a little
            OS_Sleep(1);
            continue;

        case WCM_RESULT_PROGRESS:
        case WCM_RESULT_ACCEPT:
// waits until asynchronous process response is called back
            OS_SleepThread(&gWcmThreadQ);
            break;

        case WCM_RESULT_SUCCESS:
        case WCM_RESULT_FAILURE:
// process is completed synchronously, and asynchronous processes are not performed
            return wcmResult;

        case WCM_RESULT_WMDISABLE:
            /*
* Wm library not operating on ARM7.
* You need to launch the wireless driver on ARM7 using the WVR driver before
* beginning to use WCM, but that needs to be performed within the application.
             */
            OS_Panic("ARM7 WM library is not ready.\n");

        case WCM_RESULT_NOT_ENOUGH_MEM: // this returned value will never be returned for an asynchronous function
        case WCM_RESULT_FATAL_ERROR:
        default:
            FreezeThread();
        }

// confirms asynchronous process results
        switch (gWcmResult)
        {
        case WCM_RESULT_SUCCESS:
        case WCM_RESULT_FAILURE:
            wcmResult = gWcmResult;
            break;

        case WCM_RESULT_PROGRESS:       // these process results are never passed to asynchronous process results
        case WCM_RESULT_ACCEPT:         // these process results are never passed to asynchronous process results
        case WCM_RESULT_REJECT:         // these process results are never passed to asynchronous process results
        case WCM_RESULT_WMDISABLE:      // these process results are never passed to asynchronous process results
        case WCM_RESULT_NOT_ENOUGH_MEM: // these process results are never passed to asynchronous process results
        case WCM_RESULT_FATAL_ERROR:
        default:
            FreezeThread();
        }

        break;
    }

    return wcmResult;
}

/*---------------------------------------------------------------------------*
Name: ControlCallback

Description: Callback function to receive WCM asynchronous API process result response and
irregular notifications from the WCM library.

Arguments: arg     -   Pointer to the structure indicating the notification and/or process result.

Returns: None.
 *---------------------------------------------------------------------------*/
static void ControlCallback(WCMNotice* arg)
{
    PrintSubScreen(" cb %s %s\n", gWcmNotifyText[arg->notify], gWcmResultText[arg->result]);

    switch (arg->notify)
    {
    case WCM_NOTIFY_STARTUP:
    case WCM_NOTIFY_CLEANUP:
    case WCM_NOTIFY_BEGIN_SEARCH:
    case WCM_NOTIFY_END_SEARCH:
    case WCM_NOTIFY_CONNECT:
    case WCM_NOTIFY_TERMINATE:
// asynchronous process result response in a 1:1 correspondence with requests
        gWcmResult = arg->result;
        OS_WakeupThread(&gWcmThreadQ);
        break;

    case WCM_NOTIFY_DISCONNECT:
        if (arg->parameter[1].n == 0)
        {
// result response with a 1:1 correspondence with a clear disconnect request
            gWcmResult = arg->result;
            OS_WakeupThread(&gWcmThreadQ);
        }
        else
        {
// irregular notification indicating a disconnect from the AP
            (void)OS_JamMessage(&gWcmMessageQ, LOCAL_WCM_MSG_DISCONNECTED, OS_MESSAGE_NOBLOCK);
        }
        break;

    case WCM_NOTIFY_FOUND_AP:
// irregular notification indicating found AP during auto-search
        {
            OSMessage   tempMessage;

// as notifications would take place multiple times, limited to only occurring when the message queue is empty
            if (OS_ReadMessage(&gWcmMessageQ, &tempMessage, OS_MESSAGE_NOBLOCK) == FALSE)
            {
                (void)OS_SendMessage(&gWcmMessageQ, LOCAL_WCM_MSG_FOUND_AP, OS_MESSAGE_NOBLOCK);
            }
        }
        break;

    case WCM_NOTIFY_SEARCH_AROUND:
// irregular notification indication a full loop on the specified channel during auto-search

        /*
* Ignored as this cannot occur for this sample.
         */
        break;
    }
}

/*---------------------------------------------------------------------------*
End of file
 *---------------------------------------------------------------------------*/
